# Generated by roxygen2: do not edit by hand

export(binxr_config_futures)
export(binxr_config_options)
export(binxr_config_spot)
export(cancel_fapi_trade_order)
export(cancel_fapi_trade_orders_all)
export(coerce_account_frames)
export(config_futures)
export(config_options)
export(config_spot)
export(futures_cancel_algo_order)
export(futures_cancel_all_algo_orders)
export(futures_cancel_all_orders)
export(futures_cancel_order)
export(futures_countdown_cancel_all)
export(futures_get_24hr_ticker)
export(futures_get_account)
export(futures_get_account_summary)
export(futures_get_account_trades)
export(futures_get_aggregate_trades)
export(futures_get_algo_order)
export(futures_get_algo_orders)
export(futures_get_balance)
export(futures_get_book_ticker)
export(futures_get_commission_rate)
export(futures_get_continuous_klines)
export(futures_get_exchange_info)
export(futures_get_force_orders)
export(futures_get_funding_info)
export(futures_get_funding_rate_history)
export(futures_get_historical_trades)
export(futures_get_index_constituents)
export(futures_get_index_price_klines)
export(futures_get_insurance_balance)
export(futures_get_klines)
export(futures_get_mark_price)
export(futures_get_mark_price_klines)
export(futures_get_multi_assets_mode)
export(futures_get_open_algo_orders)
export(futures_get_open_interest)
export(futures_get_open_orders)
export(futures_get_order)
export(futures_get_order_amendments)
export(futures_get_order_book)
export(futures_get_order_rate_limit)
export(futures_get_orders)
export(futures_get_position_mode)
export(futures_get_position_risk)
export(futures_get_positions)
export(futures_get_premium_index_klines)
export(futures_get_recent_trades)
export(futures_get_rpi_depth)
export(futures_get_server_time)
export(futures_get_symbol_adl_risk)
export(futures_get_ticker_price)
export(futures_get_trading_schedule)
export(futures_modify_order)
export(futures_ping)
export(futures_place_algo_order)
export(futures_place_order)
export(futures_set_leverage)
export(futures_set_margin_type)
export(futures_set_multi_assets_mode)
export(futures_set_position_mode)
export(futures_test_order)
export(get_fapi_account)
export(get_fapi_account_position_risk)
export(get_fapi_account_positions)
export(get_fapi_account_summary)
export(get_fapi_exchange_info)
export(get_fapi_klines)
export(get_fapi_mark_price)
export(get_fapi_system_time)
export(get_fapi_trade_open_orders)
export(get_fapi_trade_order)
export(get_fapi_trade_orders)
export(get_spot_mark_price)
export(options_cancel_all_orders)
export(options_cancel_all_orders_by_underlying)
export(options_cancel_order)
export(options_get_24hr_ticker)
export(options_get_account_trades)
export(options_get_commission)
export(options_get_exchange_info)
export(options_get_exercise_history)
export(options_get_exercise_records)
export(options_get_funding_flow)
export(options_get_index_price)
export(options_get_klines)
export(options_get_margin_account)
export(options_get_mark_price)
export(options_get_open_interest)
export(options_get_open_orders)
export(options_get_order)
export(options_get_order_book)
export(options_get_order_history)
export(options_get_positions)
export(options_get_recent_block_trades)
export(options_get_recent_trades)
export(options_get_server_time)
export(options_ping)
export(options_place_order)
export(place_fapi_trade_order)
export(round_price_qty)
export(set_fapi_account_leverage)
export(set_fapi_account_margin_type)
export(set_fapi_account_position_side)
export(spot_amend_order_keep_priority)
export(spot_cancel_all_orders)
export(spot_cancel_order)
export(spot_cancel_order_list)
export(spot_cancel_replace_order)
export(spot_get_24hr_ticker)
export(spot_get_account)
export(spot_get_account_trades)
export(spot_get_aggregate_trades)
export(spot_get_all_order_lists)
export(spot_get_allocations)
export(spot_get_average_price)
export(spot_get_book_ticker)
export(spot_get_commission_rates)
export(spot_get_exchange_info)
export(spot_get_execution_rules)
export(spot_get_historical_block_trades)
export(spot_get_historical_trades)
export(spot_get_klines)
export(spot_get_open_order_lists)
export(spot_get_open_orders)
export(spot_get_order)
export(spot_get_order_amendments)
export(spot_get_order_book)
export(spot_get_order_list)
export(spot_get_orders)
export(spot_get_prevented_matches)
export(spot_get_recent_trades)
export(spot_get_reference_price)
export(spot_get_reference_price_calculation)
export(spot_get_relevant_filters)
export(spot_get_rolling_window_ticker)
export(spot_get_server_time)
export(spot_get_ticker_price)
export(spot_get_trading_day_ticker)
export(spot_get_ui_klines)
export(spot_get_unfilled_order_count)
export(spot_ping)
export(spot_place_oco_order)
export(spot_place_order)
export(spot_place_order_list_oco)
export(spot_place_order_list_opo)
export(spot_place_order_list_opoco)
export(spot_place_order_list_oto)
export(spot_place_order_list_otoco)
export(spot_place_sor_order)
export(spot_test_order)
export(spot_test_sor_order)
export(util_round_price_qty)
import(data.table)
importFrom(rlang,"%||%")
