% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-spot-trade.R
\name{spot_amend_order_keep_priority}
\alias{spot_amend_order_keep_priority}
\title{Amend a Binance Spot order while keeping priority}
\usage{
spot_amend_order_keep_priority(
  symbol,
  order_id = NULL,
  orig_client_order_id = NULL,
  new_client_order_id = NULL,
  new_qty,
  config = config_spot()
)
}
\arguments{
\item{symbol}{Trading pair symbol, for example \code{"BTCUSDT"}.}

\item{order_id}{Optional exchange order ID.}

\item{orig_client_order_id}{Optional client order ID.}

\item{new_client_order_id}{Optional new client order ID after amendment.}

\item{new_qty}{New order quantity. Must be positive.}

\item{config}{A spot configuration created by \code{\link[=config_spot]{config_spot()}}.}
}
\value{
A parsed list.
}
\description{
Amend a Binance Spot order while keeping priority
}
