% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-futures-trade.R
\name{futures_get_force_orders}
\alias{futures_get_force_orders}
\title{Get Binance Futures force orders}
\usage{
futures_get_force_orders(
  symbol = NULL,
  auto_close_type = NULL,
  startTime = NULL,
  endTime = NULL,
  limit = 50,
  json_list = FALSE,
  config = config_futures()
)
}
\arguments{
\item{symbol}{Optional trading pair symbol.}

\item{auto_close_type}{Optional auto-close type.}

\item{startTime}{Optional start time in milliseconds since Unix epoch.}

\item{endTime}{Optional end time in milliseconds since Unix epoch.}

\item{limit}{Maximum number of rows to return. Must not exceed \code{100}.}

\item{json_list}{If \code{TRUE}, return the parsed list instead of a \code{data.table}.}

\item{config}{A futures configuration created by \code{\link[=config_futures]{config_futures()}}.}
}
\value{
A \code{data.table} by default, or a parsed list when \code{json_list = TRUE}.
}
\description{
Get Binance Futures force orders
}
