% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-spot-market.R
\name{spot_get_average_price}
\alias{spot_get_average_price}
\title{Get Binance Spot average price}
\usage{
spot_get_average_price(symbol, config = config_spot())
}
\arguments{
\item{symbol}{Trading pair symbol, for example \code{"BTCUSDT"}.}

\item{config}{A spot configuration created by \code{\link[=config_spot]{config_spot()}}.}
}
\value{
A parsed list.
}
\description{
Get Binance Spot average price
}
