% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-spot-market.R
\name{spot_get_order_book}
\alias{spot_get_order_book}
\title{Get Binance Spot order book}
\usage{
spot_get_order_book(
  symbol,
  limit = NULL,
  symbol_status = NULL,
  config = config_spot()
)
}
\arguments{
\item{symbol}{Trading pair symbol, for example \code{"BTCUSDT"}.}

\item{limit}{Optional order book depth limit.}

\item{symbol_status}{Optional trading status filter.}

\item{config}{A spot configuration created by \code{\link[=config_spot]{config_spot()}}.}
}
\value{
A parsed list.
}
\description{
Get Binance Spot order book
}
