% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-spot-account.R
\name{spot_get_unfilled_order_count}
\alias{spot_get_unfilled_order_count}
\title{Get Binance Spot unfilled order counts}
\usage{
spot_get_unfilled_order_count(json_list = FALSE, config = config_spot())
}
\arguments{
\item{json_list}{If \code{TRUE}, return the parsed list instead of a \code{data.table}.}

\item{config}{A spot configuration created by \code{\link[=config_spot]{config_spot()}}.}
}
\value{
A \code{data.table} by default, or a parsed list when \code{json_list = TRUE}.
}
\description{
Get Binance Spot unfilled order counts
}
