% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-spot-general.R
\name{spot_ping}
\alias{spot_ping}
\title{Test Binance Spot API connectivity}
\usage{
spot_ping(config = config_spot())
}
\arguments{
\item{config}{A spot configuration created by \code{\link[=config_spot]{config_spot()}}.}
}
\value{
An empty parsed list on success.
}
\description{
Test Binance Spot API connectivity
}
