% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-options-trade.R
\name{options_get_commission}
\alias{options_get_commission}
\title{Get Binance Options user commission}
\usage{
options_get_commission(config = config_options())
}
\arguments{
\item{config}{An options configuration created by \code{\link[=config_options]{config_options()}}.}
}
\value{
A parsed list.
}
\description{
Get Binance Options user commission
}
