% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-spot-trade.R
\name{spot_place_sor_order}
\alias{spot_place_sor_order}
\title{Place a Binance Spot SOR order}
\usage{
spot_place_sor_order(
  symbol,
  side = c("BUY", "SELL"),
  type = c("LIMIT", "MARKET"),
  quantity,
  time_in_force = NULL,
  price = NULL,
  new_client_order_id = NULL,
  strategy_id = NULL,
  strategy_type = NULL,
  iceberg_qty = NULL,
  new_order_resp_type = NULL,
  self_trade_prevention_mode = NULL,
  config = config_spot()
)
}
\arguments{
\item{symbol}{Trading pair symbol, for example \code{"BTCUSDT"}.}

\item{side}{One of \code{"BUY"} or \code{"SELL"}.}

\item{type}{SOR order type. One of \code{"LIMIT"} or \code{"MARKET"}.}

\item{quantity}{Order quantity.}

\item{time_in_force}{Optional time-in-force value.}

\item{price}{Optional limit price.}

\item{new_client_order_id}{Optional client order identifier.}

\item{strategy_id}{Optional numeric strategy identifier.}

\item{strategy_type}{Optional numeric strategy type. Must be at least \code{1000000}.}

\item{iceberg_qty}{Optional iceberg quantity.}

\item{new_order_resp_type}{Optional response type. One of \code{"ACK"}, \code{"RESULT"}, or \code{"FULL"}.}

\item{self_trade_prevention_mode}{Optional self-trade prevention mode.}

\item{config}{A spot configuration created by \code{\link[=config_spot]{config_spot()}}.}
}
\value{
A parsed list.
}
\description{
Place a Binance Spot SOR order
}
