% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-spot-market.R
\name{spot_get_rolling_window_ticker}
\alias{spot_get_rolling_window_ticker}
\title{Get Binance Spot rolling window ticker statistics}
\usage{
spot_get_rolling_window_ticker(
  symbol = NULL,
  symbols = NULL,
  windowSize = NULL,
  type = c("FULL", "MINI"),
  symbol_status = NULL,
  json_list = FALSE,
  config = config_spot()
)
}
\arguments{
\item{symbol}{Optional trading pair symbol.}

\item{symbols}{Optional character vector of trading pair symbols.}

\item{windowSize}{Optional rolling window size such as \code{"1m"}, \code{"4h"}, or \code{"7d"}.}

\item{type}{One of \code{"FULL"} or \code{"MINI"}.}

\item{symbol_status}{Optional trading status filter.}

\item{json_list}{If \code{TRUE}, return the parsed list instead of converting
array responses to \code{data.table}.}

\item{config}{A spot configuration created by \code{\link[=config_spot]{config_spot()}}.}
}
\value{
A parsed list for single-symbol requests, or a \code{data.table} for
multi-symbol requests unless \code{json_list = TRUE}.
}
\description{
Get Binance Spot rolling window ticker statistics
}
