% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-options-market.R
\name{options_get_24hr_ticker}
\alias{options_get_24hr_ticker}
\title{Get Binance Options 24hr ticker statistics}
\usage{
options_get_24hr_ticker(
  symbol = NULL,
  json_list = FALSE,
  config = config_options()
)
}
\arguments{
\item{symbol}{Optional option symbol.}

\item{json_list}{If \code{TRUE}, return the parsed list instead of a \code{data.table}.}

\item{config}{An options configuration created by \code{\link[=config_options]{config_options()}}.}
}
\value{
A \code{data.table} by default, or a parsed list when \code{json_list = TRUE}.
}
\description{
Get Binance Options 24hr ticker statistics
}
