% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-options-trade.R
\name{options_cancel_all_orders}
\alias{options_cancel_all_orders}
\title{Cancel all Binance Options orders for a symbol}
\usage{
options_cancel_all_orders(symbol, config = config_options())
}
\arguments{
\item{symbol}{Option symbol, for example \code{"BTC-200730-9000-C"}.}

\item{config}{An options configuration created by \code{\link[=config_options]{config_options()}}.}
}
\value{
A parsed list.
}
\description{
Cancel all Binance Options orders for a symbol
}
