% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-futures-trade.R
\name{futures_test_order}
\alias{futures_test_order}
\title{Test a Binance Futures order}
\usage{
futures_test_order(
  symbol,
  side = c("BUY", "SELL"),
  type = c("LIMIT", "MARKET", "STOP", "STOP_MARKET", "TAKE_PROFIT", "TAKE_PROFIT_MARKET",
    "TRAILING_STOP_MARKET"),
  quantity,
  price = NULL,
  time_in_force = NULL,
  position_side = c("BOTH", "LONG", "SHORT"),
  reduce_only = FALSE,
  stop_price = NULL,
  working_type = NULL,
  new_client_order_id = NULL,
  config = config_futures()
)
}
\arguments{
\item{symbol}{Trading pair symbol, for example \code{"ETHUSDT"}.}

\item{side}{One of \code{"BUY"} or \code{"SELL"}.}

\item{type}{Order type.}

\item{quantity}{Order quantity.}

\item{price}{Optional limit price.}

\item{time_in_force}{Optional time-in-force value. Required for \code{LIMIT}.}

\item{position_side}{One of \code{"BOTH"}, \code{"LONG"}, or \code{"SHORT"}.}

\item{reduce_only}{Whether the order is reduce-only.}

\item{stop_price}{Optional trigger price for conditional orders.}

\item{working_type}{Optional trigger source. Must only be supplied with
\code{stop_price}.}

\item{new_client_order_id}{Optional client order identifier.}

\item{config}{A futures configuration created by \code{\link[=config_futures]{config_futures()}}.}
}
\value{
A parsed list.
}
\description{
Test a Binance Futures order
}
