% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-spot-account.R
\name{spot_get_account}
\alias{spot_get_account}
\title{Get Binance Spot account information}
\usage{
spot_get_account(omit_zero_balances = FALSE, config = config_spot())
}
\arguments{
\item{omit_zero_balances}{Whether to omit zero balances from the response.}

\item{config}{A spot configuration created by \code{\link[=config_spot]{config_spot()}}.}
}
\value{
A parsed list.
}
\description{
Get Binance Spot account information
}
