% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-spot-trade.R
\name{spot_cancel_all_orders}
\alias{spot_cancel_all_orders}
\title{Cancel all open Binance Spot orders for a symbol}
\usage{
spot_cancel_all_orders(symbol, config = config_spot())
}
\arguments{
\item{symbol}{Trading pair symbol, for example \code{"BTCUSDT"}.}

\item{config}{A spot configuration created by \code{\link[=config_spot]{config_spot()}}.}
}
\value{
A parsed list by default.
}
\description{
Cancel all open Binance Spot orders for a symbol
}
