% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-spot-trade.R
\name{spot_cancel_replace_order}
\alias{spot_cancel_replace_order}
\title{Cancel and replace a Binance Spot order}
\usage{
spot_cancel_replace_order(
  symbol,
  side = c("BUY", "SELL"),
  type = c("LIMIT", "MARKET", "STOP_LOSS", "STOP_LOSS_LIMIT", "TAKE_PROFIT",
    "TAKE_PROFIT_LIMIT", "LIMIT_MAKER"),
  cancel_replace_mode = c("STOP_ON_FAILURE", "ALLOW_FAILURE"),
  time_in_force = NULL,
  quantity = NULL,
  quote_order_qty = NULL,
  price = NULL,
  cancel_new_client_order_id = NULL,
  cancel_orig_client_order_id = NULL,
  cancel_order_id = NULL,
  new_client_order_id = NULL,
  strategy_id = NULL,
  strategy_type = NULL,
  stop_price = NULL,
  trailing_delta = NULL,
  iceberg_qty = NULL,
  new_order_resp_type = NULL,
  self_trade_prevention_mode = NULL,
  cancel_restrictions = NULL,
  order_rate_limit_exceeded_mode = NULL,
  peg_price_type = NULL,
  peg_offset_value = NULL,
  peg_offset_type = NULL,
  config = config_spot()
)
}
\arguments{
\item{symbol}{Trading pair symbol, for example \code{"BTCUSDT"}.}

\item{side}{One of \code{"BUY"} or \code{"SELL"}.}

\item{type}{Order type.}

\item{cancel_replace_mode}{One of \code{"STOP_ON_FAILURE"} or \code{"ALLOW_FAILURE"}.}

\item{time_in_force}{Optional time-in-force value.}

\item{quantity}{Optional order quantity.}

\item{quote_order_qty}{Optional quote-order quantity for \code{MARKET} orders.}

\item{price}{Optional limit price.}

\item{cancel_new_client_order_id}{Optional client order ID for the cancel request.}

\item{cancel_orig_client_order_id}{Optional original client order ID to cancel.}

\item{cancel_order_id}{Optional exchange order ID to cancel.}

\item{new_client_order_id}{Optional client order identifier.}

\item{strategy_id}{Optional numeric strategy identifier.}

\item{strategy_type}{Optional numeric strategy type. Must be at least \code{1000000}.}

\item{stop_price}{Optional trigger price.}

\item{trailing_delta}{Optional trailing delta value.}

\item{iceberg_qty}{Optional iceberg quantity.}

\item{new_order_resp_type}{Optional response type. One of \code{"ACK"}, \code{"RESULT"}, or \code{"FULL"}.}

\item{self_trade_prevention_mode}{Optional self-trade prevention mode.}

\item{cancel_restrictions}{Optional cancel restriction. One of \code{"ONLY_NEW"} or \code{"ONLY_PARTIALLY_FILLED"}.}

\item{order_rate_limit_exceeded_mode}{Optional order-rate-limit behavior. One of \code{"DO_NOTHING"} or \code{"CANCEL_ONLY"}.}

\item{peg_price_type}{Optional pegged price type. One of \code{"PRIMARY_PEG"} or \code{"MARKET_PEG"}.}

\item{peg_offset_value}{Optional peg offset value.}

\item{peg_offset_type}{Optional peg offset type. Only \code{"PRICE_LEVEL"} is supported.}

\item{config}{A spot configuration created by \code{\link[=config_spot]{config_spot()}}.}
}
\value{
A parsed list.
}
\description{
Cancel and replace a Binance Spot order
}
