% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-futures-trade.R
\name{futures_get_open_orders}
\alias{futures_get_open_orders}
\alias{get_fapi_trade_open_orders}
\title{Get Binance Futures open orders}
\usage{
futures_get_open_orders(
  symbol = NULL,
  json_list = FALSE,
  config = config_futures()
)

get_fapi_trade_open_orders(symbol = NULL, config = binxr_config_futures())
}
\arguments{
\item{symbol}{Optional trading pair symbol. If \code{NULL}, returns all open
orders.}

\item{json_list}{If \code{TRUE}, return the parsed list instead of a \code{data.table}.}

\item{config}{A futures configuration created by \code{\link[=config_futures]{config_futures()}}.}
}
\value{
A \code{data.table} by default, or a parsed list when \code{json_list = TRUE}.
}
\description{
Get Binance Futures open orders
}
