% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-spot-trade.R
\name{spot_place_order_list_otoco}
\alias{spot_place_order_list_otoco}
\title{Place a Binance Spot OTOCO order list}
\usage{
spot_place_order_list_otoco(
  symbol,
  list_client_order_id = NULL,
  new_order_resp_type = NULL,
  self_trade_prevention_mode = NULL,
  working_type = c("LIMIT", "LIMIT_MAKER"),
  working_side = c("BUY", "SELL"),
  working_client_order_id = NULL,
  working_price,
  working_quantity,
  working_iceberg_qty = NULL,
  working_time_in_force = NULL,
  working_strategy_id = NULL,
  working_strategy_type = NULL,
  working_peg_price_type = NULL,
  working_peg_offset_type = NULL,
  working_peg_offset_value = NULL,
  pending_side = c("BUY", "SELL"),
  pending_quantity,
  pending_above_type = c("STOP_LOSS_LIMIT", "STOP_LOSS", "LIMIT_MAKER", "TAKE_PROFIT",
    "TAKE_PROFIT_LIMIT"),
  pending_above_client_order_id = NULL,
  pending_above_price = NULL,
  pending_above_stop_price = NULL,
  pending_above_trailing_delta = NULL,
  pending_above_iceberg_qty = NULL,
  pending_above_time_in_force = NULL,
  pending_above_strategy_id = NULL,
  pending_above_strategy_type = NULL,
  pending_above_peg_price_type = NULL,
  pending_above_peg_offset_type = NULL,
  pending_above_peg_offset_value = NULL,
  pending_below_type = c("STOP_LOSS", "STOP_LOSS_LIMIT", "TAKE_PROFIT",
    "TAKE_PROFIT_LIMIT"),
  pending_below_client_order_id = NULL,
  pending_below_price = NULL,
  pending_below_stop_price = NULL,
  pending_below_trailing_delta = NULL,
  pending_below_iceberg_qty = NULL,
  pending_below_time_in_force = NULL,
  pending_below_strategy_id = NULL,
  pending_below_strategy_type = NULL,
  pending_below_peg_price_type = NULL,
  pending_below_peg_offset_type = NULL,
  pending_below_peg_offset_value = NULL,
  config = config_spot()
)
}
\arguments{
\item{symbol}{Trading pair symbol, for example \code{"BTCUSDT"}.}

\item{list_client_order_id}{Optional order-list client ID.}

\item{new_order_resp_type}{Optional response type. One of \code{"ACK"}, \code{"RESULT"}, or \code{"FULL"}.}

\item{self_trade_prevention_mode}{Optional self-trade prevention mode.}

\item{working_type}{Working leg type. One of \code{"LIMIT"} or \code{"LIMIT_MAKER"}.}

\item{working_side}{Working leg side. One of \code{"BUY"} or \code{"SELL"}.}

\item{working_client_order_id}{Optional working-leg client ID.}

\item{working_price}{Working leg price.}

\item{working_quantity}{Working leg quantity.}

\item{working_iceberg_qty}{Optional working-leg iceberg quantity.}

\item{working_time_in_force}{Optional working-leg time-in-force.}

\item{working_strategy_id}{Optional working-leg strategy ID.}

\item{working_strategy_type}{Optional working-leg strategy type.}

\item{working_peg_price_type}{Optional working-leg peg price type.}

\item{working_peg_offset_type}{Optional working-leg peg offset type.}

\item{working_peg_offset_value}{Optional working-leg peg offset value.}

\item{pending_side}{Shared side for the pending OCO legs.}

\item{pending_quantity}{Pending leg quantity.}

\item{pending_above_type}{Type for the pending-above leg.}

\item{pending_above_client_order_id}{Optional pending-above client ID.}

\item{pending_above_price}{Optional pending-above price.}

\item{pending_above_stop_price}{Optional pending-above stop price.}

\item{pending_above_trailing_delta}{Optional pending-above trailing delta.}

\item{pending_above_iceberg_qty}{Optional pending-above iceberg quantity.}

\item{pending_above_time_in_force}{Optional pending-above time-in-force.}

\item{pending_above_strategy_id}{Optional pending-above strategy ID.}

\item{pending_above_strategy_type}{Optional pending-above strategy type.}

\item{pending_above_peg_price_type}{Optional pending-above peg price type.}

\item{pending_above_peg_offset_type}{Optional pending-above peg offset type.}

\item{pending_above_peg_offset_value}{Optional pending-above peg offset value.}

\item{pending_below_type}{Type for the pending-below leg.}

\item{pending_below_client_order_id}{Optional pending-below client ID.}

\item{pending_below_price}{Optional pending-below price.}

\item{pending_below_stop_price}{Optional pending-below stop price.}

\item{pending_below_trailing_delta}{Optional pending-below trailing delta.}

\item{pending_below_iceberg_qty}{Optional pending-below iceberg quantity.}

\item{pending_below_time_in_force}{Optional pending-below time-in-force.}

\item{pending_below_strategy_id}{Optional pending-below strategy ID.}

\item{pending_below_strategy_type}{Optional pending-below strategy type.}

\item{pending_below_peg_price_type}{Optional pending-below peg price type.}

\item{pending_below_peg_offset_type}{Optional pending-below peg offset type.}

\item{pending_below_peg_offset_value}{Optional pending-below peg offset value.}

\item{config}{A spot configuration created by \code{\link[=config_spot]{config_spot()}}.}
}
\value{
A parsed list.
}
\description{
Place a Binance Spot OTOCO order list
}
