% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-spot-account.R
\name{spot_get_order_list}
\alias{spot_get_order_list}
\title{Get a Binance Spot order list}
\usage{
spot_get_order_list(
  order_list_id = NULL,
  orig_client_order_id = NULL,
  config = config_spot()
)
}
\arguments{
\item{order_list_id}{Optional exchange order-list ID.}

\item{orig_client_order_id}{Optional order-list client ID.}

\item{config}{A spot configuration created by \code{\link[=config_spot]{config_spot()}}.}
}
\value{
A parsed list.
}
\description{
Get a Binance Spot order list
}
