% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-options-trade.R
\name{options_cancel_all_orders_by_underlying}
\alias{options_cancel_all_orders_by_underlying}
\title{Cancel all Binance Options orders by underlying}
\usage{
options_cancel_all_orders_by_underlying(underlying, config = config_options())
}
\arguments{
\item{underlying}{Underlying symbol, for example \code{"BTCUSDT"}.}

\item{config}{An options configuration created by \code{\link[=config_options]{config_options()}}.}
}
\value{
A parsed list.
}
\description{
Cancel all Binance Options orders by underlying
}
