% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-spot-trade.R
\name{spot_place_order_list_oco}
\alias{spot_place_order_list_oco}
\title{Place a Binance Spot OCO order list}
\usage{
spot_place_order_list_oco(
  symbol,
  side = c("BUY", "SELL"),
  quantity,
  above_type = c("STOP_LOSS_LIMIT", "STOP_LOSS", "LIMIT_MAKER", "TAKE_PROFIT",
    "TAKE_PROFIT_LIMIT"),
  above_client_order_id = NULL,
  above_iceberg_qty = NULL,
  above_price = NULL,
  above_stop_price = NULL,
  above_trailing_delta = NULL,
  above_time_in_force = NULL,
  above_strategy_id = NULL,
  above_strategy_type = NULL,
  above_peg_price_type = NULL,
  above_peg_offset_type = NULL,
  above_peg_offset_value = NULL,
  below_type = c("STOP_LOSS", "STOP_LOSS_LIMIT", "TAKE_PROFIT", "TAKE_PROFIT_LIMIT"),
  below_client_order_id = NULL,
  below_iceberg_qty = NULL,
  below_price = NULL,
  below_stop_price = NULL,
  below_trailing_delta = NULL,
  below_time_in_force = NULL,
  below_strategy_id = NULL,
  below_strategy_type = NULL,
  below_peg_price_type = NULL,
  below_peg_offset_type = NULL,
  below_peg_offset_value = NULL,
  list_client_order_id = NULL,
  new_order_resp_type = NULL,
  self_trade_prevention_mode = NULL,
  config = config_spot()
)
}
\arguments{
\item{symbol}{Trading pair symbol, for example \code{"BTCUSDT"}.}

\item{side}{One of \code{"BUY"} or \code{"SELL"}.}

\item{quantity}{Order quantity shared by both legs.}

\item{above_type}{Type for the above leg.}

\item{above_client_order_id}{Optional above-leg client ID.}

\item{above_iceberg_qty}{Optional above-leg iceberg quantity.}

\item{above_price}{Optional above-leg limit price.}

\item{above_stop_price}{Optional above-leg stop price.}

\item{above_trailing_delta}{Optional above-leg trailing delta.}

\item{above_time_in_force}{Optional above-leg time-in-force.}

\item{above_strategy_id}{Optional above-leg strategy ID.}

\item{above_strategy_type}{Optional above-leg strategy type.}

\item{above_peg_price_type}{Optional above-leg peg price type.}

\item{above_peg_offset_type}{Optional above-leg peg offset type.}

\item{above_peg_offset_value}{Optional above-leg peg offset value.}

\item{below_type}{Type for the below leg.}

\item{below_client_order_id}{Optional below-leg client ID.}

\item{below_iceberg_qty}{Optional below-leg iceberg quantity.}

\item{below_price}{Optional below-leg limit price.}

\item{below_stop_price}{Optional below-leg stop price.}

\item{below_trailing_delta}{Optional below-leg trailing delta.}

\item{below_time_in_force}{Optional below-leg time-in-force.}

\item{below_strategy_id}{Optional below-leg strategy ID.}

\item{below_strategy_type}{Optional below-leg strategy type.}

\item{below_peg_price_type}{Optional below-leg peg price type.}

\item{below_peg_offset_type}{Optional below-leg peg offset type.}

\item{below_peg_offset_value}{Optional below-leg peg offset value.}

\item{list_client_order_id}{Optional order-list client ID.}

\item{new_order_resp_type}{Optional response type. One of \code{"ACK"}, \code{"RESULT"}, or \code{"FULL"}.}

\item{self_trade_prevention_mode}{Optional self-trade prevention mode.}

\item{config}{A spot configuration created by \code{\link[=config_spot]{config_spot()}}.}
}
\value{
A parsed list.
}
\description{
Place a Binance Spot OCO order list
}
