% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/config.R
\name{config_options}
\alias{config_options}
\title{Create a Binance options configuration}
\usage{
config_options(
  api_key = Sys.getenv("BINX_API_KEY", unset = ""),
  secret_key = Sys.getenv("BINX_SECRET_KEY", unset = ""),
  base_url = "https://eapi.binance.com",
  recv_window = 10000L,
  timeout = 30,
  verbose = FALSE
)
}
\arguments{
\item{api_key}{character API key. Leave \code{NULL} for unsigned/public requests.}

\item{secret_key}{character Secret key. Leave \code{NULL} for unsigned/public requests.}

\item{base_url}{character Base URL for the Binance Options API.}

\item{recv_window}{integer recvWindow in milliseconds used for signed requests.}

\item{timeout}{numeric request timeout in seconds.}

\item{verbose}{logical whether to enable verbose request debugging in future helpers.}
}
\value{
A \code{binxr_config} list.
}
\description{
Create a Binance options configuration
}
\examples{
cfg <- config_options(api_key = NULL, secret_key = NULL)
cfg$product
}
