% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-futures-market.R
\name{futures_get_order_book}
\alias{futures_get_order_book}
\title{Get Binance Futures order book}
\usage{
futures_get_order_book(symbol, limit = NULL, config = config_futures())
}
\arguments{
\item{symbol}{Trading pair symbol, for example \code{"ETHUSDT"}.}

\item{limit}{Optional depth limit. One of \code{5}, \code{10}, \code{20}, \code{50}, \code{100},
\code{500}, or \code{1000}.}

\item{config}{A futures configuration created by \code{\link[=config_futures]{config_futures()}}.}
}
\value{
A parsed list.
}
\description{
Get Binance Futures order book
}
