% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-spot-trade.R
\name{spot_place_order_list_opo}
\alias{spot_place_order_list_opo}
\title{Place a Binance Spot OPO order list}
\usage{
spot_place_order_list_opo(
  symbol,
  list_client_order_id = NULL,
  new_order_resp_type = NULL,
  self_trade_prevention_mode = NULL,
  working_type = c("LIMIT", "LIMIT_MAKER"),
  working_side = c("BUY", "SELL"),
  working_client_order_id = NULL,
  working_price,
  working_quantity,
  working_iceberg_qty = NULL,
  working_time_in_force = NULL,
  working_strategy_id = NULL,
  working_strategy_type = NULL,
  working_peg_price_type = NULL,
  working_peg_offset_type = NULL,
  working_peg_offset_value = NULL,
  pending_type = c("LIMIT", "MARKET", "STOP_LOSS", "STOP_LOSS_LIMIT", "TAKE_PROFIT",
    "TAKE_PROFIT_LIMIT", "LIMIT_MAKER"),
  pending_side = c("BUY", "SELL"),
  pending_client_order_id = NULL,
  pending_price = NULL,
  pending_stop_price = NULL,
  pending_trailing_delta = NULL,
  pending_iceberg_qty = NULL,
  pending_time_in_force = NULL,
  pending_strategy_id = NULL,
  pending_strategy_type = NULL,
  pending_peg_price_type = NULL,
  pending_peg_offset_type = NULL,
  pending_peg_offset_value = NULL,
  config = config_spot()
)
}
\arguments{
\item{symbol}{Trading pair symbol, for example \code{"BTCUSDT"}.}

\item{list_client_order_id}{Optional order-list client ID.}

\item{new_order_resp_type}{Optional response type. One of \code{"ACK"}, \code{"RESULT"}, or \code{"FULL"}.}

\item{self_trade_prevention_mode}{Optional self-trade prevention mode.}

\item{working_type}{Working leg type. One of \code{"LIMIT"} or \code{"LIMIT_MAKER"}.}

\item{working_side}{Working leg side. One of \code{"BUY"} or \code{"SELL"}.}

\item{working_client_order_id}{Optional working-leg client ID.}

\item{working_price}{Working leg price.}

\item{working_quantity}{Working leg quantity.}

\item{working_iceberg_qty}{Optional working-leg iceberg quantity.}

\item{working_time_in_force}{Optional working-leg time-in-force.}

\item{working_strategy_id}{Optional working-leg strategy ID.}

\item{working_strategy_type}{Optional working-leg strategy type.}

\item{working_peg_price_type}{Optional working-leg peg price type.}

\item{working_peg_offset_type}{Optional working-leg peg offset type.}

\item{working_peg_offset_value}{Optional working-leg peg offset value.}

\item{pending_type}{Pending leg order type.}

\item{pending_side}{Pending leg side. One of \code{"BUY"} or \code{"SELL"}.}

\item{pending_client_order_id}{Optional pending-leg client ID.}

\item{pending_price}{Optional pending-leg price.}

\item{pending_stop_price}{Optional pending-leg stop price.}

\item{pending_trailing_delta}{Optional pending-leg trailing delta.}

\item{pending_iceberg_qty}{Optional pending-leg iceberg quantity.}

\item{pending_time_in_force}{Optional pending-leg time-in-force.}

\item{pending_strategy_id}{Optional pending-leg strategy ID.}

\item{pending_strategy_type}{Optional pending-leg strategy type.}

\item{pending_peg_price_type}{Optional pending-leg peg price type.}

\item{pending_peg_offset_type}{Optional pending-leg peg offset type.}

\item{pending_peg_offset_value}{Optional pending-leg peg offset value.}

\item{config}{A spot configuration created by \code{\link[=config_spot]{config_spot()}}.}
}
\value{
A parsed list.
}
\description{
Place a Binance Spot OPO order list
}
