% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-spot-trade.R
\name{spot_cancel_order}
\alias{spot_cancel_order}
\title{Cancel a Binance Spot order}
\usage{
spot_cancel_order(
  symbol,
  order_id = NULL,
  orig_client_order_id = NULL,
  new_client_order_id = NULL,
  cancel_restrictions = NULL,
  config = config_spot()
)
}
\arguments{
\item{symbol}{Trading pair symbol, for example \code{"BTCUSDT"}.}

\item{order_id}{Optional exchange order ID.}

\item{orig_client_order_id}{Optional client order ID.}

\item{new_client_order_id}{Optional client order ID for the cancel request.}

\item{cancel_restrictions}{Optional cancel restriction. One of \code{"ONLY_NEW"} or \code{"ONLY_PARTIALLY_FILLED"}.}

\item{config}{A spot configuration created by \code{\link[=config_spot]{config_spot()}}.}
}
\value{
A parsed list.
}
\description{
Cancel a Binance Spot order
}
