% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-futures-account.R
\name{futures_get_position_risk}
\alias{futures_get_position_risk}
\alias{get_fapi_account_position_risk}
\title{Get Binance Futures position risk}
\usage{
futures_get_position_risk(json_list = FALSE, config = config_futures())

get_fapi_account_position_risk(config = binxr_config_futures())
}
\arguments{
\item{json_list}{If \code{TRUE}, return the parsed list instead of a \code{data.table}.}

\item{config}{A futures configuration created by \code{\link[=config_futures]{config_futures()}}.}
}
\value{
A \code{data.table} by default, or a parsed list when \code{json_list = TRUE}.
}
\description{
Get Binance Futures position risk
}
