% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-spot-general.R
\name{spot_get_exchange_info}
\alias{spot_get_exchange_info}
\title{Get Binance Spot exchange information}
\usage{
spot_get_exchange_info(
  symbol = NULL,
  symbols = NULL,
  permissions = NULL,
  show_permission_sets = TRUE,
  symbol_status = NULL,
  config = config_spot()
)
}
\arguments{
\item{symbol}{Optional trading pair symbol.}

\item{symbols}{Optional character vector of trading pair symbols.}

\item{permissions}{Optional character vector of permission strings.}

\item{show_permission_sets}{Logical flag forwarded as \code{showPermissionSets}.}

\item{symbol_status}{Optional trading status filter.}

\item{config}{A spot configuration created by \code{\link[=config_spot]{config_spot()}}.}
}
\value{
A parsed list.
}
\description{
Get Binance Spot exchange information
}
