% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/config.R
\name{binxr_config_options}
\alias{binxr_config_options}
\title{Backward-compatible options config constructor}
\usage{
binxr_config_options(
  api_key = Sys.getenv("BINX_API_KEY", unset = ""),
  secret_key = Sys.getenv("BINX_SECRET_KEY", unset = ""),
  base = "https://eapi.binance.com",
  recvWindow = 10000L
)
}
\arguments{
\item{api_key}{character API key. Leave \code{NULL} for unsigned/public requests.}

\item{secret_key}{character Secret key. Leave \code{NULL} for unsigned/public requests.}

\item{base}{Legacy alias for \code{base_url}.}

\item{recvWindow}{Legacy alias for \code{recv_window}.}
}
\value{
A \code{binxr_config} list.
}
\description{
Backward-compatible options config constructor
}
