% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/endpoints-spot-trade.R
\name{spot_place_oco_order}
\alias{spot_place_oco_order}
\title{Place a deprecated Binance Spot OCO order}
\usage{
spot_place_oco_order(
  symbol,
  side = c("BUY", "SELL"),
  quantity,
  price,
  stop_price,
  list_client_order_id = NULL,
  limit_client_order_id = NULL,
  limit_strategy_id = NULL,
  limit_strategy_type = NULL,
  limit_iceberg_qty = NULL,
  trailing_delta = NULL,
  stop_client_order_id = NULL,
  stop_strategy_id = NULL,
  stop_strategy_type = NULL,
  stop_limit_price = NULL,
  stop_iceberg_qty = NULL,
  stop_limit_time_in_force = NULL,
  new_order_resp_type = NULL,
  self_trade_prevention_mode = NULL,
  config = config_spot()
)
}
\arguments{
\item{symbol}{Trading pair symbol, for example \code{"BTCUSDT"}.}

\item{side}{One of \code{"BUY"} or \code{"SELL"}.}

\item{quantity}{Order quantity shared by both legs.}

\item{price}{Limit price for the maker leg.}

\item{stop_price}{Stop trigger price.}

\item{list_client_order_id}{Optional order-list client ID.}

\item{limit_client_order_id}{Optional limit-leg client ID.}

\item{limit_strategy_id}{Optional limit-leg strategy ID.}

\item{limit_strategy_type}{Optional limit-leg strategy type.}

\item{limit_iceberg_qty}{Optional limit-leg iceberg quantity.}

\item{trailing_delta}{Optional shared trailing delta.}

\item{stop_client_order_id}{Optional stop-leg client ID.}

\item{stop_strategy_id}{Optional stop-leg strategy ID.}

\item{stop_strategy_type}{Optional stop-leg strategy type.}

\item{stop_limit_price}{Optional stop-limit price.}

\item{stop_iceberg_qty}{Optional stop-leg iceberg quantity.}

\item{stop_limit_time_in_force}{Optional stop-limit time-in-force.}

\item{new_order_resp_type}{Optional response type. One of \code{"ACK"}, \code{"RESULT"}, or \code{"FULL"}.}

\item{self_trade_prevention_mode}{Optional self-trade prevention mode.}

\item{config}{A spot configuration created by \code{\link[=config_spot]{config_spot()}}.}
}
\value{
A parsed list.
}
\description{
Place a deprecated Binance Spot OCO order
}
